@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 Module containing helper functions related to iris I/O methods. These contain
 reading of Cubes, and some methods to perform quality checks of the data, e.g.
@@ -7,8 +5,8 @@ reading of Cubes, and some methods to perform quality checks of the data, e.g.
 1. checking and correction of time definition
 2. number and length of dimension coordinates must match data array
 3. Longitude definition from -180 to 180 (corrected if defined on 0 -> 360 intervall)
-
 """
+
 import cf_units
 from datetime import datetime
 import iris
@@ -19,110 +17,67 @@ except ImportError:
     # old iris version installed
     from iris.experimental.equalise_cubes import equalise_attributes
 
-from numpy import datetime64, asarray, arange
-import os
-import pandas as pd
+import numpy as np
+from pathlib import Path
+from traceback import format_exc
 
 from pyaerocom import const, logger
+from pyaerocom._warnings_management import filter_warnings
 from pyaerocom.exceptions import (NetcdfError, VariableDefinitionError,
                                   FileConventionError,
                                   UnresolvableTimeDefinitionError)
 
-from pyaerocom.helpers import cftime_to_datetime64
+from pyaerocom.helpers import (cftime_to_datetime64,
+                               make_datetimeindex_from_year)
 from pyaerocom.tstype import TsType
 from pyaerocom.io.helpers import add_file_to_log
 from pyaerocom.io.fileconventions import FileConventionRead
 
-def _load_cubes_custom_multiproc(files, var_name=None, file_convention=None,
-                                 perform_fmt_checks=True, num_proc=None):
-    """Like :func:`load_cubes_custom` but faster
-
-    Uses multiprocessing module to distribute loading of multiple NetCDF files
-    into iris cube.
-
-    Parameters
-    ----------
-    file : list
-        netcdf file
-    var_name : str
-        name of variable to read
-    quality_check : bool
-        if True, then a quality check of data is performed against the
-        information provided in the filename
-    file_convention : :obj:`FileConventionRead`, optional
-        Aerocom file convention. If provided, then the data content (e.g.
-        dimension definitions) is tested against definition in file name
-    perform_fmt_checks : bool
-        if True, additional quality checks (and corrections) are (attempted to
-        be) performed.
-    num_proc : int
-        number of jobs
-
-    Returns
-    -------
-    iris.cube.CubeList
-        loaded data as Cube
-    """
-    import multiprocessing
-    from functools import partial
-    if num_proc is None:
-        num_proc = multiprocessing.cpu_count() * 2
-    func = partial(load_cube_custom,
-                   var_name=var_name, file_convention=file_convention,
-                   perform_fmt_checks=perform_fmt_checks)
-    p = multiprocessing.Pool(processes=num_proc)
-    return p.map(func, files)
-
+@filter_warnings(apply=const.FILTER_IRIS_WARNINGS,
+                 categories=[UserWarning])
 def load_cubes_custom(files, var_name=None, file_convention=None,
-                      perform_fmt_checks=True, **kwargs):
+                      perform_fmt_checks=True):
     """Load multiple NetCDF files into CubeList
 
+    Note
+    ----
+    This function does not apply any concatenation or merging of the variable
+    data in the individual files, it only loads the files into individual
+    instances of :class:`iris.cube.Cube`, which can be accessed via the
+    returned list.
+
     Parameters
     ----------
     files : list
-        netcdf file
+        list of netcdf file paths
     var_name : str
-        name of variable to read
-    quality_check : bool
-        if True, then a quality check of data is performed against the
-        information provided in the filename
+        name of variable to be imported from input files.
     file_convention : :obj:`FileConventionRead`, optional
         Aerocom file convention. If provided, then the data content (e.g.
         dimension definitions) is tested against definition in file name
     perform_fmt_checks : bool
         if True, additional quality checks (and corrections) are (attempted to
         be) performed.
-    **kwargs
-        additional keyword args that are parsed to
-        :func:`_load_cubes_custom_multiproc` in case number of input files
-        is larger than 4.
 
     Returns
     -------
-    tuple
-        2-element tuple containing:
-
-            - CubeList, containing loaded cubes
-            - list, list of filenames that were successfully loaded
+    list
+        loaded cube instances.
+    list
+        list containing all files from which the input variable could be
+        successfully loaded.
     """
-    from pyaerocom import const
     cubes = []
     loaded_files = []
-    print_where = False
-    if len(files) > 10:
-        mod = len(files) / 10
-        print_where = True
     for i, _file in enumerate(files):
-        if print_where and i%mod==0:
-            const.print_log.info(os.path.basename(_file))
         try:
-            cube = load_cube_custom(_file, var_name,
-                                    file_convention=file_convention)
+            cube = load_cube_custom(file=_file, var_name=var_name,
+                                    file_convention=file_convention,
+                                    perform_fmt_checks=perform_fmt_checks)
             cubes.append(cube)
             loaded_files.append(_file)
-        except Exception as e:
-            msg = ("Failed to load {} as Iris cube. Error: {}"
-                   .format(_file, repr(e)))
+        except Exception:
+            msg = f'Failed to load {_file}. Reason: {format_exc()}'
             const.logger.warning(msg)
 
             if const.WRITE_FILEIO_ERR_LOG:
@@ -154,26 +109,20 @@ def load_cube_custom(file, var_name=None, file_convention=None,
     iris.cube.Cube
         loaded data as Cube
     """
+    if isinstance(file, Path):
+        file = str(file) # iris load does not like PosixPath
     if perform_fmt_checks is None:
         perform_fmt_checks = const.GRID_IO.PERFORM_FMT_CHECKS
     cube_list = iris.load(file)
-
-    _num = len(cube_list)
-    if _num != 1:
-        if _num == 0:
-            raise NetcdfError('Data from file {} could not be loaded using iris'
-                              .format(file))
-        else:
-            logger.warning('File {} contains more than one variable'
-                           .format(file))
     cube = None
     if var_name is None:
         if not len(cube_list) == 1:
             vars_avail = [c.var_name for c in cube_list]
-            raise NetcdfError('Could not load single cube from {}. Please '
-                              'specify var_name. Input file contains the '
-                              'following variables: {}'.format(file,
-                                                               vars_avail))
+            raise NetcdfError(
+                f'Could not load single cube from {file}. Please '
+                f'specify var_name. Input file contains the '
+                f'following variables: {vars_avail}'
+                )
         cube = cube_list[0]
         var_name = cube.var_name
     else:
@@ -182,39 +131,45 @@ def load_cube_custom(file, var_name=None, file_convention=None,
                 cube = c
                 break
     if cube is None:
-        raise NetcdfError('Variable {} not available in file {}'.format(var_name,
-                                                                        file))
+        raise NetcdfError(
+            f'Variable {var_name} not available in file {file}'
+            )
     if perform_fmt_checks:
+        cube = _cube_quality_check(cube, file, file_convention)
+    return cube
+
+def _cube_quality_check(cube, file, file_convention=None):
+    """Perform quality check of loaded cube data
+
+    This includes the following checks (not all of them may be applicable):
+
+        - Make sure dimensionless variables have unit 1 (and not empty string)
+
+    """
+    coords = get_coord_names_cube(cube)
+    try:
+        cube = _check_cube_unitless(cube)
+    except VariableDefinitionError:
+        pass
+
+    grid_io = const.GRID_IO
+    if grid_io.CHECK_TIME_FILENAME:
         try:
-            cube = _check_var_unit_cube(cube)
-        except VariableDefinitionError:
-            pass
+            cube = _check_correct_time_dim(cube, file,  file_convention)
+        except FileConventionError:
+            const.print_log.warning('WARNING: failed to check / validate '
+                                    'time dim. using information in '
+                                    'filename. Reason: invalid file name '
+                                    'convention')
 
-        grid_io = const.GRID_IO
-        if grid_io.CHECK_TIME_FILENAME:
-            try:
-                cube = _check_correct_time_dim(cube, file,  file_convention)
-            except FileConventionError:
-                const.print_log.warning('WARNING: failed to check / validate '
-                                        'time dim. using information in '
-                                        'filename. Reason: invalid file name '
-                                        'convention')
-        else:
-            logger.warning("WARNING: Automatic check of time "
-                           "array in netCDF files is deactivated. "
-                           "This may cause problems in case "
-                           "the time dimension is not CF conform.")
-        if grid_io.CHECK_DIM_COORDS:
-            cube = check_dim_coords_cube(cube)
+    if grid_io.CHECK_DIM_COORDS:
+        cube = check_dim_coords_cube(cube)
 
-        try:
-            if grid_io.DEL_TIME_BOUNDS:
-                cube.coord("time").bounds = None
-        except Exception:
-            logger.warning("Failed to access time coordinate in GriddedData")
+    if 'time' in coords and grid_io.DEL_TIME_BOUNDS:
+        cube.coord('time').bounds = None
 
-        if grid_io.SHIFT_LONS:
-            cube = check_and_regrid_lons_cube(cube)
+    if 'longitude' in coords and grid_io.SHIFT_LONS:
+        cube = check_and_regrid_lons_cube(cube)
     return cube
 
 def check_and_regrid_lons_cube(cube):
@@ -238,21 +193,20 @@ def check_and_regrid_lons_cube(cube):
         True, if longitudes were on 0 -> 360 and have been rolled, else
         False
     """
-    from pyaerocom import print_log
-    try:
-        if cube.coord("longitude").points.max() > 180:
-            logger.info("Rolling longitudes to -180 -> 180 definition")
-            cube = cube.intersection(longitude=(-180, 180))
-    except Exception as e:
-        print_log.warning('Failed to roll longitudes: {}'.format(repr(e)))
+    if cube.coord("longitude").points.max() > 180:
+        const.print_log.info(
+            'Rearranging longitude dimension from 0 -> 360 '
+            'definition to -180 -> 180 definition'
+            )
+        cube = cube.intersection(longitude=(-180, 180))
     return cube
 
 def check_dim_coord_names_cube(cube):
 
     from pyaerocom import const
-    coords = dict(lon = const.COORDINFO.lon,
-                  lat = const.COORDINFO.lat,
-                  time = const.COORDINFO.time)
+    coords = dict(lon = const.COORDINFO['lon'],
+                  lat = const.COORDINFO['lat'],
+                  time = const.COORDINFO['time'])
 
     for coord in cube.dim_coords:
         cv, cs, cn = coord.var_name, coord.standard_name, coord.long_name
@@ -306,153 +260,89 @@ def check_dim_coords_cube(cube):
     cube = check_dim_coord_names_cube(cube)
     return cube
 
-def _check_var_unit_cube(cube):
+def _check_cube_unitless(cube):
+    """Make sure unit in Cube is 1 if variable is dimensionless
+    """
     var = cube.var_name
     if not var in const.VARS:
-        raise VariableDefinitionError('No such pyaerocom default variable: {}'
-                                      .format(cube.var_name))
+        raise VariableDefinitionError(
+            f'No such pyaerocom default variable: {cube.var_name}'
+            )
 
-    u = cube.units
-    if isinstance(u, str):
-        u = cf_units.Unit(u)
-    if str(const.VARS[var].units) == '1' and u.is_unknown():
-        const.print_log.info('Overwriting unit {} in cube {} with value "1"'
-                             .format(str(u), var))
+    unit = cf_units.Unit(cube.units)
+    if str(const.VARS[var].units) == '1' and unit.is_unknown():
         cube.units = cf_units.Unit('1')
     return cube
 
-def check_time_coordOLD(cube, ts_type, year):
-    """Method that checks the time coordinate of an iris Cube
-
-    This method checks if the time dimension of a cube is accessible and
-    according to the standard (i.e. fully usable). It only checks, and does not
-    correct. For the latter, please see :func:`correct_time_coord`.
+def _get_info_from_filename(file, file_convention=None):
+    """Load meta-information from filename
 
     Parameters
     ----------
-    cube : Cube
-        cube containing data
-    ts_type : str
-        temporal resolution of data (e.g. "hourly", "daily"). This information
-        is e.g. encrypted in the filename of a NetCDF file and may be
-        accessed using :class:`pyaerocom.io.FileConventionRead`
-    year : int
-        interger specifying year of observation, e.g. 2017
-
-    Returns
-    -------
-    bool
-        True, if time dimension is ok, False if not
     """
-
-    ok = True
-    ts_type = TsType(ts_type)
-    test_idx = [0,1,2,7] #7, since last accessible index in a 3hourly dataset of one day is 7
-    try:
-        try:
-            t = cube.coord("time")
-        except Exception:
-            raise AttributeError("Cube does not contain time dimension")
-        if not isinstance(t, iris.coords.DimCoord):
-            raise AttributeError("Time is not a DimCoord instance")
-        try:
-            cftime_to_datetime64(0, cfunit=t.units)
-        except Exception:
-            raise ValueError("Could not convert time unit string")
-# =============================================================================
-#         tres_np = TSTR_TO_NP_TD[ts_type]
-#         conv = TSTR_TO_NP_DT[ts_type]
-# =============================================================================
-        tres_np = ts_type.timedelta64_str
-        conv = ts_type.datetime64_str_str
-
-        base = datetime64("{}-01-01 00:00:00".format(year)).astype(conv)
-        test_datenums = asarray(test_idx)
-        ts_nominal = base + test_datenums.astype(tres_np)
-        dts_nominal = ts_nominal[1:] - ts_nominal[:-1]
-        ts_values = cftime_to_datetime64(t[test_idx].points, cfunit=t.units).astype(conv)
-        dts_values = ts_values[1:] - ts_values[:-1]
-        if not all(ts_values == ts_nominal):
-            raise ValueError("Time match error, nominal dates for test array"
-                             "%s (unit=%s): %s\nReceived values after "
-                             "conversion: %s"
-                             %(test_datenums, t.units.origin,
-                               ts_nominal, ts_values))
-        elif not all(dts_values == dts_nominal):
-            raise ValueError("Time match error, time steps for test array"
-                             "%s (unit=%s): %s\nReceived values after "
-                             "conversion: %s"
-                             %(test_datenums, t.units.origin,
-                               dts_nominal, dts_values))
-    except Exception as e:
-        logger.warning("Invalid time dimension.\n"
-                       "Error message: {}".format(repr(e)))
-        ok = False
-    return ok
-
-def make_datetimeindex_from_year(freq, year):
-    """Create pandas datetime index
-
-    Parameters
-    ----------
-    freq : str
-        pandas frequency str
-    year : int
-        year
-
-    Returns
-    -------
-    pandas.DatetimeIndex
-        index object
-    """
-    start = datetime64("{}-01-01 00:00:00".format(year))
-    stop = datetime64("{}-12-31 23:59:59".format(year))
-    idx = pd.date_range(start=start, end=stop,
-                        freq=freq)
-
-    return idx
+    if file_convention is None:
+        file_convention = FileConventionRead(from_file=file)
+    return file_convention.get_info_from_file(file)
 
 def _check_correct_time_dim(cube, file, file_convention=None):
-    if file_convention is None:
-        try:
-            file_convention = FileConventionRead(from_file=file)
-        except Exception:
-            pass
+    """Check if time dimension in input Cube is correct
 
-    if not isinstance(file_convention, FileConventionRead):
+    Note
+    -----
+    Needs information about time dimension encoded in filename, since the
+    check is done against what is specified in the filename. E.g. AeroCom
+    format
+    Parameters
+    ----------
+    cube : iris.cube.Cube
+        loaded Cube instance, for which time dimension is supposed to be
+        checked
+    file : str
+        path to file from which the Cube was imported
+    file_convention : FileConventionRead
+        file naming convention specifying how time dimension information is
+        encoded in the filenames.
 
-        raise FileConventionError('Unknown file convention: {}'
-                                  .format(file_convention))
+    """
+    finfo = _get_info_from_filename(file, file_convention)
 
-    finfo = file_convention.get_info_from_file(file)
-    try:
-        ts_type = TsType(finfo['ts_type'])
-    except Exception:
-        raise FileConventionError('Invalid ts_type in file: {}'
-                                  .format(ts_type))
+    ts_type = TsType(finfo['ts_type'])
     year = finfo['year']
 
     if not const.MIN_YEAR <= year <= const.MAX_YEAR:
         raise FileConventionError('Invalid year in file: {}'.format(year))
-    try:
-        check_time_coord(cube, ts_type, year)
-    except UnresolvableTimeDefinitionError as e:
-        raise UnresolvableTimeDefinitionError(repr(e))
-    except Exception:
-        msg = ("Invalid time dimension coordinate in file {}. "
-               .format(os.path.basename(file)))
-        logger.warning(msg)
-        if const.GRID_IO.CORRECT_TIME_FILENAME:
-            logger.warning("Attempting to correct time coordinate "
-                           "using information in file name")
-            try:
-                cube = correct_time_coord(cube,
-                                          ts_type=finfo["ts_type"],
-                                          year=finfo["year"])
-            except Exception:
-                pass
-        if const.WRITE_FILEIO_ERR_LOG:
-            add_file_to_log(file, 'Invalid time dimension')
+    elif year == 9999:
+        const.print_log.info(
+            'Cannot compare NetCDF time dimension for climatological data '
+            '(9999 in filename). Skipping this check.')
+    else:
+        try:
+            check_time_coord(cube, ts_type, year)
+        except UnresolvableTimeDefinitionError as e:
+            raise UnresolvableTimeDefinitionError(repr(e))
+        except Exception as e:
+            msg = (f'Invalid time dimension coordinate in file:\n{file}.\n'
+                   f'Error: repr({e})\n')
+            logger.warning(msg)
+            if const.GRID_IO.CORRECT_TIME_FILENAME:
+                add_msg = ('Attempting to correct time coordinate using '
+                           'information in file name')
+                msg += add_msg
+                logger.info(add_msg)
+                try:
+                    cube = correct_time_coord(cube,
+                                              ts_type=finfo["ts_type"],
+                                              year=finfo["year"])
+                except Exception:
+                    add_msg = (
+                        f'Unable to correct time dimension using the '
+                        f'information provided in the file name. Error:\n'
+                        f'{format_exc()}.\n\nThe file will be imported regardless!'
+                        )
+                    msg += add_msg
+                    const.print_log.warning(msg)
+            if const.WRITE_FILEIO_ERR_LOG:
+                add_file_to_log(file, msg)
     return cube
 
 def _check_leap_year(num, num_per, ts_type):
@@ -487,14 +377,14 @@ def check_time_coord(cube, ts_type, year):
     """
     if isinstance(ts_type, str):
         ts_type = TsType(ts_type)
-    try:
-        t = cube.coord("time")
-    except Exception:
+    if not 'time' in get_coord_names_cube(cube):
         raise AttributeError("Cube does not contain time dimension")
-    if not isinstance(t, iris.coords.DimCoord):
+    tdim = cube.coord("time")
+
+    if not isinstance(tdim, iris.coords.DimCoord):
         raise AttributeError("Time is not a DimCoord instance")
     try:
-        cftime_to_datetime64(0, cfunit=t.units)
+        cftime_to_datetime64(0, cfunit=tdim.units)
     except Exception:
         raise ValueError("Could not convert time unit string")
 
@@ -503,18 +393,19 @@ def check_time_coord(cube, ts_type, year):
     tidx = make_datetimeindex_from_year(freq, year)
 
     num_per = len(tidx)
-    num = len(t.points)
+    num = len(tdim.points)
 
     if not num == num_per:
         if tidx[0].is_leap_year:
             if not _check_leap_year(num, num_per, ts_type):
-                raise UnresolvableTimeDefinitionError('Expected {} timestamps but '
-                                                  'data has {}'
-                                                  .format(len(tidx), num))
+                raise UnresolvableTimeDefinitionError(
+                    f'Expected {len(tidx)} timestamps but data has {num}'
+                    )
+
         else:
-            raise UnresolvableTimeDefinitionError('Expected {} timestamps but '
-                                                  'data has {}'
-                                                  .format(len(tidx), num))
+            raise UnresolvableTimeDefinitionError(
+                    f'Expected {len(tidx)} timestamps but data has {num}'
+                    )
 
     # ToDo: check why MS is not working for period conversion
     if freq == 'MS':
@@ -525,14 +416,60 @@ def check_time_coord(cube, ts_type, year):
     per1 = tidx[-1].to_period(freq)
 
     # first and last timestamp in data
-    t0, t1 = cftime_to_datetime64([t.points[0], t.points[-1]], cfunit=t.units)
+    t0, t1 = cftime_to_datetime64([tdim.points[0], tdim.points[-1]],
+                                  cfunit=tdim.units)
 
     if not per0.start_time <= t0 <= per0.end_time:
-        raise ValueError('First timestamp of data {} does not lie in first '
-                         'period: {}'.format(t0, per0))
+        raise ValueError(
+            f'First timestamp of data {t0} does not lie in first period: {per0}'
+            )
     elif not per1.start_time <= t1 <= per1.end_time:
-        raise ValueError('Last timestamp of data {} does not lie in last '
-                         'period: {}'.format(t1, per1))
+        raise ValueError(
+            f'Last timestamp of data {t1} does not lie in end period: {per1}'
+            )
+
+def get_dim_names_cube(cube):
+    return [c.name() for c in cube.dim_coords]
+
+def get_coord_names_cube(cube):
+    return [c.name() for c in cube.coords()]
+
+def _get_time_index_cube(cube):
+    """
+    Get array index of time dimension for input Cube
+
+    Parameters
+    ----------
+    cube : iris.cube.Cube
+        data cube.
+
+    Raises
+    ------
+    IndexError
+        if index cannot be retrieved (e.g. data does not contain time
+        dimension).
+
+    Returns
+    -------
+    int
+
+    """
+    dim_names = get_dim_names_cube(cube)
+    if 'time' in dim_names:
+        return dim_names.index('time')
+
+    idx_miss = []
+    if cube.ndim != len(dim_names): #one dimension is missing
+        for idx in range(len(cube.shape)):
+            coords = cube.coords(contains_dimension=idx,
+                                 dim_coords=True)
+            if len(coords)==0:
+                idx_miss.append(idx)
+    if len(idx_miss) == 1:
+        return idx_miss[0]
+
+    raise IndexError(f'Failed to identify data index of time dimension in '
+                      f'cube {repr(cube)}')
 
 def correct_time_coord(cube, ts_type, year):
     """Method that corrects the time coordinate of an iris Cube
@@ -543,10 +480,10 @@ def correct_time_coord(cube, ts_type, year):
         cube containing data
     ts_type : TsType or str
         temporal resolution of data (e.g. "hourly", "daily"). This information
-        is e.g. encrypted in the filename of a NetCDF file and may be
+        is e.g. encoded in the filename of a NetCDF file and may be
         accessed using :class:`pyaerocom.io.FileConventionRead`
     year : int
-        interger specifying start year, e.g. 2017
+        integer specifying start year, e.g. 2017
 
     Returns
     -------
@@ -554,46 +491,58 @@ def correct_time_coord(cube, ts_type, year):
         the same instance of the input cube with corrected time dimension axis
 
     """
-    tindex_cube = None
-    dim_lens = []
+    tindex_cube = _get_time_index_cube(cube)
+    coords = get_coord_names_cube(cube)
+
     if isinstance(ts_type, str):
         ts_type = TsType(ts_type)
-    for i, coord in enumerate(cube.dim_coords):
-        dim_lens.append(len(coord.points))
-        if coord.name() == 'time':
-            tindex_cube = i
-    if tindex_cube is None:
-        if cube.ndim != len(cube.dim_coords): #one dimension is missing
-            for idx, dim_len in enumerate(cube.shape):
-                if not dim_len in dim_lens: #candidate
-                    tindex_cube = idx
-    if tindex_cube is None:
-        raise NetcdfError('Failed to identify data index of time dimension in '
-                          'cube {}'.format(repr(cube)))
+
     tres_str = ts_type.cf_base_unit
     conv = ts_type.datetime64_str
-    tunit_str = '%s since %s-01-01 00:00:00' %(tres_str, year)
+    tunit_str = f'{tres_str} since {year}-01-01 00:00:00'
     num = cube.shape[tindex_cube]
 
     tunit = cf_units.Unit(tunit_str, calendar=cf_units.CALENDAR_STANDARD)
     tres_np = ts_type.timedelta64_str #TSTR_TO_NP_TD[ts_type]
-    base = datetime64("%s-01-01 00:00:00" %year).astype(conv)
-    times = base + arange(0, num, 1).astype(tres_np)
+    base = np.datetime64(f'{year}-01-01 00:00:00').astype(conv)
+    times = base + np.arange(0, num, 1).astype(tres_np)
+
     # see this thread https://github.com/matplotlib/matplotlib/issues/2259/
     times_dt = times.astype("datetime64[s]").astype(datetime)
-#    timestamps = datetime64(str(year)) +
-    time_nums = [tunit.date2num(t) for t in times_dt]
-    tcoord = iris.coords.DimCoord(time_nums, standard_name='time', units=tunit)
 
-    #tcoord_dim = cube.coord_dims('time')
-    try:
+    time_nums = [tunit.date2num(t) for t in times_dt]
+
+    pd_freq = ts_type.to_pandas_freq()
+    num_expected = len(make_datetimeindex_from_year(pd_freq, year))
+    num_inferred = len(time_nums)
+    if not num_inferred == num_expected:
+        raise UnresolvableTimeDefinitionError(
+            f'expected {num_expected} timestamps for {year} and '
+            f'freq {ts_type} but got {num_inferred}')
+    tcoord = iris.coords.DimCoord(time_nums,
+                                  standard_name='time',
+                                  units=tunit)
+    if 'time' in coords:
         cube.remove_coord('time')
-    except Exception:
-        pass
+
     cube.add_dim_coord(tcoord, tindex_cube)
     cube.attributes['timedim-corrected'] = True
     return cube
 
+def _check_correct_dtypes_timedim_cube_list(cubes):
+
+    try:
+        dtypes = np.unique([cube.coord('time').points.dtype for cube in cubes])
+    except iris.exceptions.CoordinateNotFoundError:
+        return False
+    corrected = False
+    if len(dtypes) > 1:
+        corrected = True
+        for cube in cubes:
+            new = cube.coord('time').points.astype(float)
+            cube.coord('time').points = new
+    return corrected
+
 def concatenate_iris_cubes(cubes, error_on_mismatch=True):
     """Concatenate list of :class:`iris.Cube` instances cubes into single Cube
 
@@ -645,16 +594,15 @@ def concatenate_iris_cubes(cubes, error_on_mismatch=True):
     equalise_attributes(cubes)
     #unify time units
     iris.util.unify_time_units(cubes)
-
     #now concatenate the cube list to one cube
-    cubes_concat = iris._concatenate.concatenate(cubes, error_on_mismatch)
+    try:
+        cubes_concat = iris._concatenate.concatenate(cubes, error_on_mismatch)
+    except Exception as e: #
+        if _check_correct_dtypes_timedim_cube_list(cubes):
+            cubes_concat = iris._concatenate.concatenate(cubes,
+                                                         error_on_mismatch)
+        else:
+            raise
 
     return cubes_concat[0]
 
-if __name__== "__main__":
-    import pyaerocom as pya
-
-    r = pya.io.ReadGridded('BCC-CUACE_HIST')
-
-    d = r.read_var('zg')
-    print(d)
