@@ -1,11 +1,9 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 Methods to convert different standards of vertical coordinates
 
 For details see here:
 
-    http://cfconventions.org/Data/cf-conventions/cf-conventions-1.0/build/apd.html
+http://cfconventions.org/Data/cf-conventions/cf-conventions-1.0/build/apd.html
 
 Note
 ----
@@ -453,8 +451,7 @@ class AltitudeAccess(object):
                 self[coord] = d[coord]
             else:
                 try:
-                    self[coord] = d.search_other(coord,
-                                                 require_same_shape=False)
+                    self[coord] = d.search_other(coord)
                     print('Adding coord {}'.format(coord))
                 except Exception:
                     all_ok = False
